import {
  Modal, ModalService, Button, Date as ZdDate, Select, Form,
} from '@zeedhi/common';
import {
  IDictionary, IEventParam,
  Loader, Metadata, I18n, Config, Http, dayjs as moment,
} from '@zeedhi/core';
import { PmeMask } from '@/config';

export class Pme06600SubsPratoCard {
  private modalSubsPrato!: Modal;
  
  public appController = Loader.getInstance('AppController');
  
  public utilities = Loader.getInstance('Utilities');
  
  private elaboracaoCardapio = Loader.getInstance('Pme00600ElaboracaoCardapio');
  
  private gridEditableController = Loader.getInstance('GridEditableController');
  
  public mensagem = '';
  
  private dtProcessa = '';
  
  private formIsValid(form: Form) {
    let valid = false;
    const row: IDictionary = form.value;
    
    row.TAMACNIVEL = this.utilities.calcTamacNivel(row.NIVEL, 'produto');
    
    if (form.validate()) {
      const dtDe = row.DTDE;
      const dtAte = row.DTATE;

      if (this.utilities.dateIsValid(dtDe, dtAte)) valid = true;
      else this.utilities.showAlert('alertError', 'pme::dataIniMaior', 'error');
    }
    return valid;
  }
  
  public validaDataProcessamento() {
    const dtIni = Metadata.getInstance<ZdDate>('DTDE');
    const dtFin = Metadata.getInstance<ZdDate>('DTATE');
    const dataProcessamento = this.utilities.dtProcessFormatada;
    let isValid = false;
    
    if (dtIni.value !== '' && dtFin.value !== '') {
      isValid = true;
      
      if (moment(dtIni.value, 'DD/MM/YYYY') < dataProcessamento || moment(dtFin.value, 'DD/MM/YYYY') < dataProcessamento) {
        this.utilities.showAlert('alertError', I18n.translate('pme::dtProcessamentoForaPeriodo', { PARAM1: dataProcessamento }), 'error');
        isValid = false;
      } else if (moment(dtIni.value, 'DD/MM/YYYY') > moment(dtFin.value, 'DD/MM/YYYY')) {
        this.utilities.showAlert('alertError', 'pme::dataIniMaior', 'error');
        isValid = false;
      }
    }
    
    return isValid;
  }
  
  public validPratoDestino() {
    const pratoDest = Metadata.getInstance<Select>('PRATO_DESTINO').value;
    
    if (pratoDest.IDRECEITA === 'N') {
      this.utilities.showAlert('Alert', 'pme::pratoDestSemReceita', 'error');
      pratoDest.value = '';
    }
  }
  
  public setFilterTpCardapio({ component }: IEventParam<Select>) {
    const central = Metadata.getInstance<Select>('CENTRAL').value;

    const filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: central.CDFILIAL,
      },
      {
        name: 'NRPASTATCARD',
        operator: '=',
        value: null,
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public setFilterPrato({ component }: IEventParam<Select>) {
    const nivelMax = (Config.masks.plateMask as PmeMask).nivelMax;
  
    const filter = [
      {
        name: 'NIVEL',
        operator: '=',
        value: nivelMax,
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public setFilterServ({ component }: IEventParam<Select>) {
    const central = Metadata.getInstance<Select>('CENTRAL').value;
    const grupServ = Metadata.getInstance<Select>('GRUPOSERVICO');
    const grupServValue = grupServ.value ? grupServ.value : 'T';
  
    const filter = [
      {
        name: 'P_CDFILIAL',
        operator: '=',
        value: central.CDFILIAL,
      },
      {
        name: 'P_CDGRUPSERV',
        operator: '=',
        value: grupServValue,
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public setFilterAgreg({ component }: IEventParam<Select>) {
    const central = Metadata.getInstance<Select>('CENTRAL').value;
  
    const filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: central.CDFILIAL,
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public async createModal() {
    this.modalSubsPrato = await ModalService.createFromJson('modals/pme06600_substituicao_de_pratos_erros', true);
  }
  
  public cancelModal() {
    this.modalSubsPrato.hide();
    this.mensagem = '';
  }
  
  private async validAndOpenModal() {
    if (this.modalSubsPrato) {
      this.modalSubsPrato.show();
    } else {
      this.createModal().then(() => {
        this.modalSubsPrato.show();
      });
    }
  }
  
  public confirmSubsPrato({ component }: IEventParam<Button>) {
    const form: Form = Metadata.getInstance('formSubsPrato');
    const row = form.value;
    const dataProcessamento = this.dtProcessa;
    const dtProcessIsValid = this.validaDataProcessamento();

    if (form.validate() && dtProcessIsValid) {
      this.utilities.activeLoadingButton(component);
      Http.post('/card/confirmSubsPrato', { requestType: 'Row', row }).then((response) => {
        const result = response.data.dataset.result;
        if (result) {
          if (result.length === 1) {
            const dtCarDia = result[0].DTCARDIAPME;
            const descProd = `${result[0].CDPRODUTO} - ${result[0].NMPRODUTO}`;
            const descTipoCard = `${result[0].CDTIPOCARD} - ${result[0].NMTIPOCARD}`;
            const descServ = `${result[0].CDSERVFILI} - ${result[0].NMSERVICO}`;
            if (result[0].NRERRO === 0) this.utilities.showAlert('alertError', I18n.translate('pme::opBloqProdutoInativoReceita', { PARAM1: descProd }), 'error');
            if (result[0].NRERRO === 1) {
              this.utilities.showAlert('alertError', I18n.translate('pme::comensaisZerados', {
                PARAM1: dtCarDia, PARAM2: descTipoCard, PARAM3: descServ,
              }), 'error');
            }
            if (result[0].NRERRO === 2) this.utilities.showAlert('Alert', 'pme::pratoDestSemReceita', 'error');
            if (result[0].NRERRO === 3) this.utilities.showAlert('Alert', 'pme::pratoDestSemReceitaRevisada', 'error');
            if (result[0].NRERRO === 4) this.utilities.showAlert('alertError', I18n.translate('pme::dtProcessamentoForaPeriodo', { PARAM1: dataProcessamento }), 'error');
            if (result[0].NRERRO === 5) this.utilities.showAlert('Alert', 'pme::pratoDestInativo', 'error');
            if (result[0].NRERRO === 6) this.utilities.showAlert('Alert', 'pme::pratoOrigemCardSelecionado', 'warning');
          } else if (result.length > 1) {
            result.forEach((posicao: IDictionary) => {
              const dtCarDia = posicao.DTCARDIAPME;
              const descProd = `${posicao.CDPRODUTO} - ${posicao.NMPRODUTO}`;
              const descTipoCard = `${posicao.CDTIPOCARD} - ${posicao.NMTIPOCARD}`;
              const descServ = `${posicao.CDSERVFILI} - ${posicao.NMSERVICO}`;
              if (posicao.NRERRO === 0) {
                const texto = `${I18n.translate('pme::opBloqProdutoInativoReceita', { PARAM1: descProd })}`;
                this.mensagem += `${texto} ${'<br>'} ${'<br>'}`;
              }
              if (posicao.NRERRO === 1) {
                const texto = `${I18n.translate('pme::comensaisZerados', {
                  PARAM1: dtCarDia, PARAM2: descTipoCard, PARAM3: descServ,
                })}`;
                this.mensagem += `${texto} ${'<br>'} ${'<br>'}`;
              }
              if (posicao.NRERRO === 2) {
                const texto = `${I18n.translate('pme::pratoDestSemReceita')}`;
                this.mensagem += `${texto} ${'<br>'} ${'<br>'}`;
              }
              if (posicao.NRERRO === 3) {
                const texto = `${I18n.translate('pme::pratoDestSemReceitaRevisada')}`;
                this.mensagem += `${texto} ${'<br>'} ${'<br>'}`;
              }
              if (posicao.NRERRO === 4) {
                const texto = `${I18n.translate('pme::pratoDestInativo')}`;
                this.mensagem += `${texto} ${'<br>'} ${'<br>'}`;
              }
              if (posicao.NRERRO === 5) {
                const texto = `${I18n.translate('pme::opBloqProdutoInativoReceita', { PARAM1: dataProcessamento })}`;
                this.mensagem += `${texto} ${'<br>'} ${'<br>'}`;
              }
              if (posicao.NRERRO === 6) {
                const texto = `${I18n.translate('pme::pratoOrigemCardSelecionado')}`;
                this.mensagem += `${texto} ${'<br>'} ${'<br>'}`;
              }
            });
            this.validAndOpenModal();
          } else if (result.length === 0) this.utilities.showAlert('Alert', 'pme::salvoComSucesso', 'success');
        }
      }).finally(() => {
        this.utilities.disableLoadingButton(component);
      });
    }
  }
  
  public clearFieldsCentral({ component }: IEventParam<Select>) {
    const cardapio = Metadata.getInstance<Select>('TIPOCARD');
    const servico = Metadata.getInstance<Select>('SERVICO');
    const unidade = Metadata.getInstance<Select>('UNIDADE');
    
    if (component.name === 'CENTRAL') {
      cardapio.setValue('');
      servico.setValue('');
      unidade.setValue('');
    }
  }
  
  public setValueCentral() {
    const selectCentral = Metadata.getInstance<Select>('CENTRAL');
    selectCentral.setValue(this.appController.getCdCentral());
  }
}
